---
title: "Implementation Plan: Refactor InfoService to use Endpoint instances"
issue: 97
author: "@khive-implementer"
created: "2025-05-18"
status: "Completed"
---

# Implementation Plan: Refactor InfoService to use Endpoint instances

## 1. Overview

This implementation plan outlines the approach for refactoring the `InfoService`
to use `Endpoint` instances (obtained via `match_endpoint`) for all external API
calls, as specified in Issue #97 and TDS-80.md. The refactoring will ensure that
`Endpoint` correctly uses `AsyncAPIClient` internally.

## 2. Requirements

- Refactor `InfoService` to use `Endpoint` instances for all external API calls
- Ensure `Endpoint` correctly uses `AsyncAPIClient` internally
- Maintain a minimalistic style in the service implementation
- Review `ReaderService` for any similar applicable changes
- Ensure all new/modified code includes tests with >80% coverage

## 3. Design Approach

### 3.1 InfoService Refactoring

The current `InfoService` implementation makes direct API calls to external
services. We will refactor it to:

1. Use lazy-loaded `Endpoint` instances obtained via `match_endpoint`
2. Properly handle request formatting for each provider
3. Ensure proper resource cleanup

### 3.2 Implementation Strategy

1. Analyze the current `InfoService` implementation
2. Identify all external API calls
3. Replace direct API calls with calls through `Endpoint` instances
4. Update error handling to properly handle endpoint-related errors
5. Ensure proper resource cleanup in the `close()` method
6. Write comprehensive tests to verify the refactored implementation

## 4. Implementation Details

### 4.1 InfoService Changes

The `InfoServiceGroup` class will be refactored to:

- Store endpoint instances as instance variables (`_perplexity`, `_exa`,
  `_openrouter`)
- Initialize these endpoints lazily when they are first used
- Use the `match_endpoint` function to obtain the appropriate endpoint for each
  provider
- Format requests according to each provider's requirements
- Handle errors appropriately
- Ensure proper resource cleanup

### 4.2 Testing Strategy

We will create comprehensive tests that:

- Verify that endpoints are correctly initialized
- Test successful API calls through endpoints
- Test error handling for various failure scenarios
- Verify proper resource cleanup
- Include integration tests that verify end-to-end functionality

## 5. Implementation Steps

1. Refactor `InfoServiceGroup` to use lazy-loaded endpoints
2. Update the search methods to use endpoints
3. Update the consult method to use endpoints
4. Ensure proper resource cleanup in the `close()` method
5. Create comprehensive tests for the refactored implementation
6. Verify test coverage meets the >80% requirement

## 6. Risks and Mitigations

| Risk                            | Mitigation                                                                |
| ------------------------------- | ------------------------------------------------------------------------- |
| Breaking existing functionality | Comprehensive test coverage to ensure all functionality works as expected |
| Resource leaks                  | Ensure proper cleanup in the `close()` method                             |
| Performance impact              | Monitor performance during testing                                        |

## 7. Testing Plan

### 7.1 Unit Tests

- Test endpoint initialization
- Test successful API calls
- Test error handling
- Test resource cleanup

### 7.2 Integration Tests

- Test end-to-end functionality for search and consult operations

## 8. Acceptance Criteria

- All tests pass
- Test coverage is >80%
- Code follows project style guidelines
- No resource leaks

## 9. References

- Issue #97
- TDS-80.md
- Issue #80
- Issue #81
